import visit from 'unist-util-visit';
import { VFile } from 'vfile';
import rule from 'unified-lint-rule';
import grayMatter from 'gray-matter';
import Ajv from 'ajv';
import AjvErrors from 'ajv-errors';
import YAMLSourceMap from 'yaml-source-map';
import yamlLib from 'yaml';
import path from 'path';
import fs from 'fs';
import VMessage, { VFileMessage } from 'vfile-message';

// JSON schema for front-matter validation
const playgroundExampleSchema = {
    type: 'object',
    properties: {
        examples: {
            type: 'array',
            items: {
                type: 'object',
                properties: {
                    name: { type: 'string' },
                    label: { type: 'string'},
                    description: { type: 'string'},
                },
                required: ['name', 'label', 'description'],
                additionalProperties: false,
                errorMessage: {
                    required: {
                        name: "Playground example front matter is invalid. 'name' is a required property.",
                        label: "Playground example front matter is invalid. 'label' is a required property.",
                        description: "Playground example front matter is invalid. 'description' is a required property."
                    },
                }
            },
        },
    },
    required: ['examples'],
};

const ajv = new Ajv({ allErrors: true });
AjvErrors(ajv);
const validateSchema = ajv.compile(playgroundExampleSchema);

function checkForExampleFolder(node, file, examples, cwd) {
    const fileDirectory = path.resolve(file.path ? path.dirname(file.path) : cwd);
    const examplesDirectory = path.join(fileDirectory, 'examples');

    // Check if specific files exist based on front matter content
    examples.forEach((example: any) => {
        const folderName = example.name;
        const folderPath = path.join(examplesDirectory, folderName);

        try {
            if (!fs.existsSync(folderPath)) {
                file.message(
                    `Folder '${folderName}' is missing but referenced in the playground example front matter.`,
                    node
                );
                return;
            }
            const filesInExampleDir = fs.readdirSync(folderPath);
            if(filesInExampleDir.length === 0) {
                file.message(
                    `Folder '${folderPath}' is empty. Add the respective example files.`,
                    node
                );
            }
        } catch (err) {
            file.message(`Error checking example folder: '${folderPath}'`);
        }
    });
}

function validateFrontMatterInMarkdown(tree: any, file: VFile, cwd: string): void {
    visit(tree, ['yaml'], (node: any) => {
        const fileContent = file.toString();

        //Reg-ex to check if the example front matter exists or no.
        const frontMatterMatch = fileContent.match(/---\n([\s\S]+?)\n---/);
        if (!frontMatterMatch) {
            return;
        }

        const { data } = grayMatter(fileContent);
        if(!validateSchema(data)) {
            if (validateSchema.errors.length > 0) {
                const sourceMap = new YAMLSourceMap();
                const document = sourceMap.index(
                    yamlLib.parseDocument(fileContent, {
                        keepCstNodes: true,
                    }),
                );
                // To calculate the exact line of error in front-matter we need the following logic
                validateSchema.errors.forEach((error) => {
                    const schemaPath = error.dataPath.split('/').filter(function (e) {
                        return String(e).trim();
                    });
                    const fileCounter = sourceMap.lookup(schemaPath, document);
                    const line = fileCounter && fileCounter.start ? fileCounter.start.line : 0;
                    const column = fileCounter && fileCounter.start ? fileCounter.start.col : 0;

                    const errorMessage: VFileMessage = new VMessage(
                        error.message,
                        { line: line, column: column },
                        node
                    );
                    errorMessage.fatal = true;
                    file.messages.push(errorMessage);
                });
            }
            return;
        }

        checkForExampleFolder(node, file, data?.examples, cwd);
    });
}

export = rule('sfdocs-lint:remark-lint-markdown-frontmatter-playground', validateFrontMatterInMarkdown);